#!/bin/sh -e

if [ ! -x ./work/fproxy ]; then
  echo "compile first!"
  exit
fi

BINS="fproxy_r fproxy-cache_add fproxy fproxy-dashboard"
HBINS="ssl-client-wrapper fproxy-client-wrapper ssl-server-wrapper"

rm -Rf ./work/deb ./work/deb-h
mkdir ./work/deb ./work/deb-h

# place files
mkdir -p ./work/deb/usr/bin ./work/deb/usr/share/fproxy ./work/deb-h/usr/bin
for fn in $BINS; do
  cp -p ./work/$fn ./work/deb/usr/bin/$fn
done
for fn in $HBINS; do
  cp -p ./work/$fn ./work/deb-h/usr/bin/$fn
done
tar -C share -c -f - . | tar -x -f - -C ./work/deb/usr/share/fproxy

# place controls
VERSION=`head -1 ./src/version.txt | tr -d -`
ARCH=`dpkg --print-architecture`
SIZE=`du -k -s ./work/deb | awk '{print $1}'`
SIZEH=`du -k -s ./work/deb-h | awk '{print $1}'`
VER1=`dpkg-query -W -f='${Version}' libc6`
SSLNAME=`ldd ./work/fproxy-client-wrapper | grep -F "libssl.so." | awk '{print $3}'`
SSLDEB=`(dpkg -S $SSLNAME || dpkg -S /usr$SSLNAME) | cut -d: -f1 -s`
VER2=`dpkg-query -W -f='${Version}' $SSLDEB`
VER3=`dpkg-query -W -f='${Version}' libfccl`
ZLIBNAME=`ldd ./work/fproxy | grep -F "libz.so." | awk '{print $3}'`
ZLIBDEB=`(dpkg -S $ZLIBNAME || dpkg -S /usr$ZLIBNAME) | cut -d: -f1 -s`
VER4=`dpkg-query -W -f='${Version}' $ZLIBDEB`
mkdir ./work/deb/DEBIAN
( cd ./work/deb && md5deep -r -l usr) > ./work/deb/DEBIAN/md5sums
cat > ./work/deb/DEBIAN/control <<EOF
Package: fproxy
Version: $VERSION
Architecture: $ARCH
Maintainer: firk <firk@cantconnect.ru>
Installed-Size: $SIZE
Depends: libc6 (>= $VER1), libfccl (>= $VER3), $ZLIBDEB, fproxy-helpers (>= $VERSION)
Recommends: openssl
Section: net
Priority: optional
Multi-Arch: foreign
Homepage: https://dev.m1089.ru/fproxy
Description: multipurpose http(s) proxy server
EOF

mkdir ./work/deb-h/DEBIAN
( cd ./work/deb-h && md5deep -r -l usr) > ./work/deb-h/DEBIAN/md5sums
cat > ./work/deb-h/DEBIAN/control <<EOF
Package: fproxy-helpers-$SSLDEB
Version: $VERSION
Architecture: $ARCH
Maintainer: firk <firk@cantconnect.ru>
Installed-Size: $SIZEH
Depends: libc6 (>= $VER1), $SSLDEB (>= $VER2), libfccl (>= $VER3)
Provides: fproxy-helpers (= $VERSION)
Recommends: openssl
Section: net
Priority: optional
Multi-Arch: foreign
Homepage: https://dev.m1089.ru/fproxy
Description: multipurpose http(s) proxy server - ssl/tls helpers
EOF

fakeroot dpkg-deb --build ./work/deb ./work
fakeroot dpkg-deb --build ./work/deb-h ./work

rm -Rf ./work/deb ./work/deb-h

echo "DONE"

