#!/bin/sh -e

TDIR="$1"
TUID="$2"
TGID="$3"

if [ -z "$TDIR" ]; then
  echo "Usage: $0 /target/dir [<daemon_uid> <daemon_gid>]" 1>&2
  exit 1
fi

SECURE_INSTALL=
if [ -n "$TUID" ] || [ -n "$TGID" ]; then
  if [ -z "$TUID" ] || [ -z "$TGID" ]; then
    echo "Usage: $0 /target/dir [<daemon_uid> <daemon_gid>]" 1>&2
    exit 1
  fi
  if [ `id -u` != 0 ]; then
    echo "Need to run as root if you want to setup uid/gid" 1>&2
    exit 1
  fi
  SECURE_INSTALL=1
fi

if [ -e "$TDIR" ]; then
  if [ ! -d "$TDIR" ]; then
    echo "target path $TDIR already exists and is not a directory" 1>&2
    exit 1
  fi
  if ls -A "$TDIR" | read q; then
    echo "target path $TDIR already exists and is not empty" 1>&2
    exit 1
  fi
fi

if [ -x /bin/realpath ] || [ -x /usr/bin/realpath ]; then
  PWD0=`dirname $0`
  SDIR=`realpath "$PWD0"`
else
  PWD0=`dirname $0`
  OLDPWD=`pwd`
  cd "$PWD0"
  SDIR=`pwd`
  cd "$OLDPWD"
fi


echo "Creating working directory structure"
mkdir -p "$TDIR"

RDIR="$SDIR/rules_example"
cat $SDIR/rules_.txt $RDIR/rules_block.txt > $TDIR/rules_block.txt
cat $SDIR/rules_.txt $RDIR/rules_cache.txt > $TDIR/rules_cache.txt
cat $SDIR/rules_.txt $RDIR/rules_cookie.txt > $TDIR/rules_cookie.txt
cat $SDIR/rules_.txt $RDIR/rules_raw.txt > $TDIR/rules_raw.txt
cat $SDIR/rules_.txt $RDIR/rules_save.txt > $TDIR/rules_save.txt
cat $SDIR/rules_.txt $RDIR/rules_sniff.txt > $TDIR/rules_sniff.txt
cp $RDIR/rules_route.txt $TDIR/
cp $RDIR/proxy.sh $RDIR/proxy.conf $TDIR/
cp $RDIR/proxy2.sh $RDIR/proxy2.conf $TDIR/

cd "$TDIR"
mkdir cache_data cache_real saved_data saved_real cert-pin cert-pin/certs cert-pin/queue cert-pin/cache dyn-certs internal log pem proxy_temp tmp
chmod 700 . *
chmod 600 rules_*.txt *.conf
echo 0 > dyn-cert-serial
echo 0 > mincache.time
echo 0 > offline.flag

openssl genrsa -out pem/cakey.pem 4096
openssl genrsa -out pem/proxykey.pem 4096
openssl req -x509 -new -key pem/cakey.pem -days 10000 -out pem/cacert.pem -subj '/O=local proxy/OU=local proxy CA/CN=local proxy'

chmod 400 pem/*.pem

if [ -n "$SECURE_INSTALL" ]; then
  echo "Securing installation for $TUID:$TGID"
  chown -R "0:$TGID" .
  chmod 755 .
  chmod 770 cache_data cache_real saved_data saved_real internal dyn-certs proxy_temp log
  chmod 775 tmp

  chmod 750 cert-pin pem
  chmod 770 cert-pin/*
  chmod 440 pem/*

  chmod 440 rules_*.txt
  chmod 660 dyn-cert-serial
  chmod 440 mincache.time offline.flag
fi


echo "directory $TDIR (`pwd`) prepared"
echo "created certificate `pwd`/pem/cacert.pem that should be installed as trusted root CA into your browser"

echo "DONE"
