/*
 * This file is part of fproxy - multipurpose http(s) proxy server
 * helpers/conn_open.h - open a connection through various paths
 * Copyright (c) 2023 firk (firk@cantconnect.ru)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */
#ifndef INCLUDED_FPROXY_HELPERS_CONN_OPEN_H
#define INCLUDED_FPROXY_HELPERS_CONN_OPEN_H

typedef enum { proxy_none=0, proxy_http, proxy_https, proxy_socks4, proxy_socks5, proxy_r } proxy_type;

typedef struct proxy_desc {
  proxy_type type;
  uint32     ip;
  uint16     port;
  char const *auth;
} proxy_desc;

typedef struct {
  char const *wrapper;
  /* possible vallues:
     "/path/to/binary"
     "unix:/path/to/socket"
     "tcp:1.2.3.4:1234"
     NULL (only for non-ssl) */
  proxy_desc via;
} connection_path;

typedef void (*conn_open_log_fn)(void *cbdata, char const *fmt, ...);
extern int conn_open_via(uint32 addr, uint16 port, char const *hostname, proxy_desc const *via, uint32 flags, conn_open_log_fn logfn, void *cbdata, int verbose);

/* return: -1=parse error; -2=out of memory */
extern int parse_proxyspec(proxy_desc *r, char const *b);
extern void free_proxyspec(proxy_desc *r);

#endif
