/*
 * This file is part of fproxy - multipurpose http(s) proxy server
 * helpers/lst_ssl.h - libsimpletcp-like functions for using ssl-wrappers
 * Copyright (c) 2018, 2022, 2023 firk (firk@cantconnect.ru)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */


typedef struct { /* this struct is used only internally; it is black box */
  int fd, meta, nonblock, is_remote;
  unsigned int pending_err;
  unsigned int pos;
  unsigned char buf[1024];
  unsigned int hpos;
  unsigned char hbuf[3];
  int cstr;
  char * cbuf;
  size_t cbufsz, cbufpos;
} lst_conn_open_ssl_ctx;

typedef struct {
  char const *wrapper;
  struct proxy_desc const *proxy;
  int raw_fd;
  uint32 addr;
  uint16 port;
  char const *domain;
  char const *sni;
  char const *alpn;    /* only when use_ssl */
  uint16 minver;       /* only when use_ssl */
  char const * cacert; /* only when use_ssl */
  char * cbuf;
  size_t cbufsz;
  uint32 flags;
  int use_ssl;
} lst_conn_open_ssl_req;

extern int lst_conn_open_ssl(lst_conn_open_ssl_req *req, char *errmsg, size_t errmsgsize, lst_conn_open_ssl_ctx *ctx);
extern int lst_wait_conn_open_ssl(lst_conn_open_ssl_ctx *ctx, char *errmsg, size_t errmsgsize);
extern int lst_conn_accept_ssl_ex(int raw_fd, uint16 minver, char const *cacert, char const *cakey, char const *serverkey, char *errmsg, size_t errmsgsize, uint32 flags, char const *default_hostname);
#define lst_conn_accept_ssl(raw_fd,invvar,cacert,caky,serverkey,errmsg,errmsgsize,flags) lst_conn_accept_ssl_ex(raw_fd,invvar,cacert,caky,serverkey,errmsg,errmsgsize,flags,NULL)
