/*
 * This file is part of fproxy - multipurpose http(s) proxy server
 * helpers/nclient.h - main header file for fproxy-client-wrapper
 * Copyright (c) 2023 firk (firk@cantconnect.ru)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */
#ifndef INCLUDED_FPROXY_HELPERS_NCLIENT_H
#define INCLUDED_FPROXY_HELPERS_NCLIENT_H

typedef struct {
  uint32      mask;
  uint16      minver;
  int         legacy_server;
  char const  *client_cert_file;
  char const  *client_pkey_file;
  char const  *calist_file;
  int         enforce_verify;
  int         verify_depth;
  uchar       alpn[256];
  size_t      alpn_size;
  int         use_ssl;
  int         preconnected_fd;
  uint32      ip;
  uint16      port;
  char const  *addrstr;
  char const  *domain; /* use for resolving ip if ip not given */
  char const  *sni; /* always verified by nclient.c:valid_hostname() if not NULL; to be used for SNI */
  int         do_dump_certs;
  proxy_desc  via;
} conn_options;

extern void raw_fail(char const * fmt, ...);
extern void set_process_namef(char const *fmt, ...);
extern int init_raw_connection(conn_options *o);

extern void init_ssl(conn_options *o);
extern void free_ssl(void);
extern SSL * start_ssl_connection(conn_options *o, int * r_fd);
extern void end_ssl_connection(SSL * ssl, int fd, int wait_reply);
extern void get_ssl_cert(conn_options *o);
extern void send_ssl_bytes(SSL * ssl, int fd, unsigned char const * b, size_t n);
extern size_t recv_ssl_bytes(SSL * ssl, int fd, unsigned char * b, size_t n, int * p_eof);

#endif
